/*
 *  Copyright (C) 2004 Jonathan Derque <jolapache@free.fr>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include <stdlib.h>
#include <string.h>
#include "buffer.h"

#define INITIAL_SIZE 80
#define INCREMENT 20


struct buffer_s
{
  char *buffer;
  int size;
  int current_position;
};

buffer
buffer_create ()
{
  buffer b;
  b = (buffer) malloc (sizeof (struct buffer_s));
  if (!b)
    return NULL;
  b->size = INITIAL_SIZE;
  b->buffer = (char *) malloc (b->size * sizeof (char));
  if (!b->buffer)
    {
      free (b);
      return NULL;
    }
  b->current_position = 0;
  *(b->buffer + b->current_position) = '\0';
}

void
buffer_destroy (buffer b)
{
  if (!b)
    return;
  free (b->buffer);
  free (b);
  return;
}

static int
buffer_realloc (buffer b)
{
  char *ptr;
  if (!b)
    return 0;
  ptr = (char *) realloc (b->buffer, b->size + INCREMENT);
  if (!ptr)
    return 0;
  b->buffer = ptr;
  b->size += INCREMENT;
  return 1;
}

int
buffer_insert_char (buffer b, char c)
{
  int ret_val = 1;

  if (!b)
    return 0;
  if (b->current_position >= b->size - 1)
    ret_val = buffer_realloc (b);
  if (!ret_val)
    return 0;
  *(b->buffer + b->current_position) = c;
  b->current_position++;
  *(b->buffer + b->current_position) = '\0';
  return 1;
}

int
buffer_get_string (buffer b, char **s)
{
  *s = NULL;
  if (!b)
    return 0;
  *s = strdup (b->buffer);
  return 1;
}

int
buffer_clear (buffer b)
{
  if (!b)
    return 0;
  b->current_position = 0;
  *(b->buffer + b->current_position) = '\0';
  return 1;
}
