/*
 *  Copyright (C) 2004 Jonathan Derque <jolapache@free.fr>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <reponcal/reponcal.h>
#include "buffer.h"
#include "tokenizer.h"
#include "calculator.h"

extern int done;
extern reponcal rpc;

int
exec_command (char *command)
{
  int ret_val;
  if (!command || strcmp (command, "") == 0)
    return 1;

  if (strcmp (command, "exit") == 0)
    {
      done = 1;
    }
  else if (strcmp (command, "+") == 0)
    {
      reponcal_calculate (rpc, "add");
    }
  else if (strcmp (command, "-") == 0)
    {
      reponcal_calculate (rpc, "sub");
    }
  else if (strcmp (command, "*") == 0)
    {
      reponcal_calculate (rpc, "mul");
    }
  else if (strcmp (command, "/") == 0)
    {
      reponcal_calculate (rpc, "div");
    }
  else
    {
      ret_val = reponcal_calculate (rpc, command);
      if (!ret_val)
        printf ("%s: command not found\n", command);
    }
  return 0;
}


int
parse_string (char *string)
{
  tokenizer t;
  char *token;
  char *command;
  if (!string)
    return 1;
  if (*string == '\0')
    return 0;
  t = tokenizer_create (string, DELIM);
  while (tokenizer_has_next (t) && !done)
    {
      tokenizer_next_token (t, &token);
      if (isdigit (*token))
        {
          double *d;
          d = (double *) malloc (sizeof (double));
          if (!d)
            done = 1;
          *d = strtod (token, &command);
          reponcal_push (rpc, "double", d);
        }
      else
        {
          command = token;
        }
      exec_command (command);
      free (token);
    }

  tokenizer_destroy (t);

  return 0;
}
