/*
 *  Copyright (C) 2004 Jonathan Derque <jolapache@free.fr>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include <stdio.h>
#include <reponcal/reponcal.h>
#include "terminal_ui.h"
#include "buffer.h"
#include "calculator.h"

#define PROMPT "reponcal> "

buffer b;
extern int done;
extern reponcal rpc;

/* calls reponcal_print and print the stack content */
void
print_stack (reponcal rpc)
{
  char **strings;
  char **strings2;
  int size;

  strings = reponcal_print (rpc);
  strings2 = strings;
  reponcal_size (rpc, &size);
  while (size)
    {
      printf ("%s\n", *strings);
      free (*strings);
      strings++;
      size--;
    }
  free (strings2);
}

int
terminal_ui_init ()
{
  b = buffer_create ();
  return (int) b;
}


int
terminal_ui_main ()
{
  char c;
  char *buf;

  printf ("%s", PROMPT);
  while (done == 0)
    {
      c = getc (stdin);
      if (c != '\n')
        {
          buffer_insert_char (b, c);
        }
      else
        {
          buffer_get_string (b, &buf);
          buffer_clear (b);
          parse_string (buf);
          free (buf);
          print_stack (rpc);
          printf ("%s", PROMPT);
        }
    }
  return 0;
}

int
terminal_ui_end ()
{
  buffer_destroy (b);
  return 0;
}
