/*
 *  Copyright (C) 2004 Jonathan Derque <jolapache@free.fr>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tokenizer.h"

struct tokenizer_s
{
  char *string;
  char *delimiters;
  char *current_position;
};

tokenizer
tokenizer_create (char *string, char *delimiters)
{
  tokenizer t;
  t = (tokenizer) malloc (sizeof (struct tokenizer_s));
  if (!t)
    return NULL;
  t->string = string;
  t->delimiters = delimiters;
  tokenizer_init (t);
  return t;
}

void
tokenizer_destroy (tokenizer t)
{
  if (t)
    free (t);
}

int
tokenizer_init (tokenizer t)
{
  if (!t)
    return 0;
  t->current_position = t->string;
  return 1;
}

int
tokenizer_set_delimiters (tokenizer t, char *delimiters)
{
  if (!t)
    return 0;
  t->delimiters = delimiters;
  return 1;
}

int
tokenizer_has_next (tokenizer t)
{
  char *s_ptr;
  if (!t)
    return 0;
  s_ptr = t->current_position;
  while (*s_ptr && strchr (t->delimiters, *s_ptr))
    s_ptr++;
  return *s_ptr != '\0';
}

int
tokenizer_next_token (tokenizer t, char **token)
{
  char *s_ptr;
  if (!t)
    return 0;
  while (*(t->current_position)
         && strchr (t->delimiters, *(t->current_position)))
    t->current_position++;
  s_ptr = t->current_position;
  while (*s_ptr && !strchr (t->delimiters, *s_ptr))
    s_ptr++;
  *token =
    (char *) strndup (t->current_position, s_ptr - t->current_position);
  t->current_position = s_ptr;
  return 1;
}
